IDEAL
MODEL	LARGE,C
P386
JUMPS


INCLUDE	"NC4.INC"



DATASEG


PUBLIC	ScrWidth, StartAddr


LABEL	CRTCValues	WORD
	DW	05F00h
	DW	04F01h
	DW	05002h
	DW	08203h
	DW	05404h
	DW	08005h
	DW	00D06h
	DW	03E07h
	DW	00008h
	DW	04109h
	DW	0EA10h
	DW	0AC11h
	DW	0DF12h
	DW	02813h
	DW	00014h
	DW	0E715h
	DW	00616h
	DW	0E317h

CRTCVALCNT = (($-CRTCValues)/2)


ScrWidth  DW	80			; screen width in bytes

StartAddr  DW	0			; start address of active page





CODESEG


PUBLIC	SetNC4Mode
PUBLIC	DrawPixel
PUBLIC	ReadPixel
PUBLIC	SetScrStart
PUBLIC	WaitDE
PUBLIC	WaitVR




PROC	SetNC4Mode	FAR	ModeNum : WORD
USES	si,di

	cld				; just to make sure...

	cmp	[ModeNum],NC4MODE_320x200
	jb	@@err			; legal mode number?
	cmp	[ModeNum],NC4MODE_320x480
	ja	@@err

	mov	ax,0013h		; set mode 13h (standard 320x200x256)
	int	10h

	cli				; disable interrupts for safety

	mov	dx,03C4h		; Sequencer address register
	mov	ax,0604h		; no Chain Four, Sequential mem, >64k
	out	dx,ax

	mov     dx,03D4h		; CRT Controller address register
	mov     al,14h
	out     dx,al
	inc     dx
	in      al,dx
	and     al,0BFh
	out     dx,al

	dec     dx
	mov     al,17h
	out     dx,al
	inc     dx
	in      al,dx
	or      al,40h
	out     dx,al
	dec	dx

	cmp	[ModeNum],NC4MODE_320x200	; does the display mode have
	je	@@400				;  400 scan lines?
	cmp	[ModeNum],NC4Mode_320x400
	je	@@400


	mov	dx,03C4h		; now set up 480 scan lines mode
	mov	ax,0100h
	out	dx,ax

	mov	al,0E3h
	mov	dx,3C2h
	out	dx,al

	mov	dx,03C4h
	mov	ax,0300h
	out	dx,ax

	mov	dx,03D4h
	mov	al,11h
	out	dx,al
	inc	dx
	in	al,dx
	and	al,7Fh
	out	dx,al
	dec	dx

	mov	si,offset CRTCValues
	mov	cx,CRTCVALCNT

	rep	outsw


@@400:	sti

	cmp	[ModeNum],NC4MODE_320x200
	je	@@DS			; use double-scan?
	cmp	[ModeNum],NC4MODE_320x240
	je	@@DS

	mov	dx,03D4h
	mov	al,9
	out	dx,al			; double-scan off
	inc	dx
	in	al,dx
	and	al,0E0h
	out	dx,al

@@DS:	mov	dx,03D4h		; use double-scan
	mov	al,13h
	out	dx,al
	inc	dx
	mov	ax,40
	out	dx,al

	xor	di,di
	mov	ax,0A000h
	mov	es,ax
	mov	cx,16384
	xor	eax,eax
	rep	stosd

@@err:
	ret
ENDP




PROC	DrawPixel	FAR	X : WORD, Y : WORD, Color : BYTE

	mov	ax,[Y]
	mul	[ScrWidth]
	mov	bx,[X]			; BX = start addr in disp mem
	shr	bx,2
	add	bx,ax
	add	bx,[StartAddr]

	mov	al,02h
	mov	cl,[byte X]		; Sequencer, index 2 (map mask)
	and	cl,3			;  selects the plane(s) where writes
	mov	ah,1			;  will take place
	shl	ah,cl
	mov	dx,03C4h
	out	dx,ax

	mov	ax,0A000h
	mov	es,ax
	mov	al,[Color]
	mov	[es:bx],al

	ret
ENDP



PROC	ReadPixel	FAR	X : WORD, Y : WORD

	mov	ax,[Y]
	mul	[ScrWidth]
	mov	bx,[X]
	shr	bx,2
	add	bx,ax
	add	bx,[StartAddr]

	mov	al,4
	mov	ah,[byte X]		; Graphics Controller, index 2 (read
	and	ah,3			;  map) selects the plane where data
	mov	dx,03CEh		;  will be read from
	out	dx,ax

	mov	ax,0A000h
	mov	es,ax

	mov	al,[es:bx]

	ret
ENDP



PROC	SetScrStart	FAR	StartAddress : WORD

	mov	dx,03D4h
	mov	al,0Dh			; index 0Dh in CRT Controller controls
	mov	ah,[byte StartAddress]	;  low byte of the start address of
	out	dx,ax			;  the visible display page

	mov	al,0Ch			; index 0Ch controls the high byte
	mov	ah,[byte StartAddress+1]
	out	dx,ax

; Start address should be set before Vertical Retrace (preferably during
;  Display Enable to make sure Vertical Retrace doesn't occur too soon) and
;  will be read into VGA's internal registers in the beginning of next
;  Vertical Retrace.

	ret
ENDP




PROC	WaitDE		FAR		; waits for Display Enable

	mov	dx,03DAh
@@w:	in	al,dx
	test	al,1
	jnz	@@w			; Display Enable is active low

	ret
ENDP



PROC	WaitVR		FAR		; waits for Vertical Retrace

	mov	dx,03DAh
@@w:	in	al,dx
	test	al,8
	jz	@@w

	ret
ENDP



END